/*<ATGCOPYRIGHT>
 * Copyright (C) 2004 Art Technology Group, Inc.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from Art Technology Group.  This notice must be
 * included on all copies, modifications and derivatives of this
 * work.
 *
 * Art Technology Group (ATG) MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. ATG SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * "Dynamo" is a trademark of Art Technology Group, Inc.
 </ATGCOPYRIGHT>*/

package com.atg.ant.taskdefs;

import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.TaskContainer;


/**
 * ForEachBuildNum is an ant task that simply counts the build numbers
 * in between the base build number and the current build number.  The
 * counter is exposed as a property named build.counter.
 *
 */
public class ForEachBuildNum extends Task implements TaskContainer {

    private Vector mTasks = new Vector();
    public void addTask (Task task) {
	mTasks.addElement(task);
    }


    private int mBaseBuildNum;
    /**
     * setter for the baseBuildNum attribute
     */
    public void setBaseBuildNum (String pBaseBuildNum) {
	try {
	    mBaseBuildNum = java.lang.Integer.parseInt(pBaseBuildNum);
	}
	catch (NumberFormatException nfe) {
	    // not a number, so assume that the variable was not declared
	    // hence, we will assume a default base build number
	    //
	    mBaseBuildNum = -1;
	}
    }

    private int mCurrBuildNum;
    /**
     * setter for the currBuildNum attribute
     */
    public void setCurrBuildNum (int pCurrBuildNum) {
	mCurrBuildNum = pCurrBuildNum;
    }


    /**
     * execute the task
     * @throws BuildException
     */
    public void execute ()
	throws BuildException
    {
	// base build number not set, so let's assume it is one less
	// than the current build number
	//
	if (mBaseBuildNum == -1)
	    mBaseBuildNum = mCurrBuildNum - 1;

	// expose a property to record where we started the count
	// since we handle a default case when it is not passed in
	//
	getProject().setProperty("build.start", mBaseBuildNum + "");
	for (int i = mBaseBuildNum+1; i <= mCurrBuildNum; i++) {

	    // set a build.counter
	    //
	    getProject().setProperty("build.counter", i + "");

	    // execute the tasks
	    //
	    for (int j=0; j < mTasks.size(); j++) {
		((Task) mTasks.get(j)).perform();
	    }
	}
    }
}
