/*<ATGCOPYRIGHT>
 * Copyright (C) 2004 Art Technology Group, Inc.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from Art Technology Group.  This notice must be
 * included on all copies, modifications and derivatives of this
 * work.
 *
 * Art Technology Group (ATG) MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. ATG SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * "Dynamo" is a trademark of Art Technology Group, Inc.
 </ATGCOPYRIGHT>*/

package com.atg.ant.taskdefs;

import ise.antelope.tasks.Variable;

import java.io.File;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Ant;

/**
 * IterateAnt is an ant task which will invoke the ant task for each module
 * 
 * It takes two required attributes
 * <li>modules list of modules
 * <li>target target to invoke
 *  
 */
public class IterateAnt extends Task
{
	protected String mBasedir;
	protected String mPropertyname;
	protected String mDirlist;
	protected String mTarget;

	/**
	 * @param pBasedir The basedir to set.
	 */
	public void setBasedir(String pBasedir)
	{
		mBasedir = pBasedir;
	}

	/**
	 * @param pDirlist The dirlist to set.
	 */
	public void setDirlist(String pDirlist)
	{
		mDirlist = pDirlist;
	}

	/**
	 * @param pPropertyname The propertyname to set.
	 */
	public void setPropertyname(String pPropertyname)
	{
		mPropertyname = pPropertyname;
	}

	/**
	 * setter for the target attribute
	 */
	public void setTarget(String pTarget)
	{
		mTarget = pTarget;
	}

	/**
	 * execute the task
	 * 
	 * @throws BuildException
	 */
	public void execute() throws BuildException
	{
		// instantiate the ant task that we will re-use
		//
		Ant ant = new Ant();
		ant.setProject(getProject());
		ant.setTarget(mTarget);
		Variable var = new Variable();
		var.setProject(getProject());
		var.setName(mPropertyname);
		String oldValue = getProject().getProperty(mPropertyname);
		
		// for every module...
		//
		StringTokenizer strtok = new StringTokenizer(mDirlist);
		String value = null;
		while (strtok.hasMoreTokens())
		{
			// reset the dir and invoke ant
			value = strtok.nextToken();
			var.setValue(value);
			var.perform();
			ant.setDir(new File(mBasedir + "/" + value));
			ant.perform();
		}
		
		var.setValue(oldValue);
		var.perform();
	}
}
