/*
 * <ATGCOPYRIGHT> Copyright (C) 2004 Art Technology Group, Inc. All Rights
 * Reserved. No use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Art Technology Group. This
 * notice must be included on all copies, modifications and derivatives of this
 * work.
 * 
 * Art Technology Group (ATG) MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. ATG SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * "Dynamo" is a trademark of Art Technology Group, Inc.
 */

package com.atg.ant.taskdefs;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

/**
 * @author Thomas Woodard
 */
public class StripWhitespaceFromJsp extends Task
{

  protected static Hashtable mRegexTable = new Hashtable(10);

  protected static String NEW_LINE = "\n";

  public static Pattern getPattern(String s)
  {
    Object obj = mRegexTable.get(s);
    Pattern pattern = null;
    if (obj == null)
    {
      try
      {
        pattern = Pattern.compile(s);
        mRegexTable.put(s, pattern);
      }
      catch (PatternSyntaxException e)
      {
        throw new BuildException(e);
      }
    }
    else
    {
      pattern = (Pattern) obj;
    }

    return pattern;
  }

  protected boolean mFailOnError = true;

  protected ArrayList mFileList;

  protected String mFilePath;

  protected Vector mFilesets = new Vector();

  protected RandomAccessFile mRAF;

  public StripWhitespaceFromJsp()
  {
  }

  /**
   * Adds a set of files to strip.
   * 
   * @param pSet
   *          a set of files to strip
   */
  public void addConfiguredFileset(FileSet pSet)
  {
    mFilesets.addElement(pSet);
  }

  public void execute() throws BuildException
  {
    for (int i = 0; i < mFilesets.size(); i++)
    {
      FileSet fs = (FileSet) mFilesets.elementAt(i);
      DirectoryScanner ds = null;
      try
      {
        ds = fs.getDirectoryScanner(getProject());
      }
      catch (BuildException e)
      {
        if (mFailOnError || !e.getMessage().endsWith(" not found."))
        {
          throw e;
        }
        else
        {
          log("Warning: " + e.getMessage());
          continue;
        }
      }

      ds.scan();
      trim(ds.getBasedir(), ds.getIncludedFiles());
    }
  }

  /**
   * @return Returns the fileList.
   */
  public ArrayList getFileList()
  {
    return mFileList;
  }

  /**
   * @return Returns the filePath.
   */
  public String getFilePath()
  {
    return mFilePath;
  }

  /**
   * @return Returns the filesets.
   */
  public Vector getFilesets()
  {
    return mFilesets;
  }

  /**
   * @return Returns the failOnError.
   */
  public boolean isFailOnError()
  {
    return mFailOnError;
  }

  /**
   * @param pFailOnError
   *          The failOnError to set.
   */
  public void setFailOnError(boolean pFailOnError)
  {
    mFailOnError = pFailOnError;
  }

  /**
   * @param pFileList
   *          The fileList to set.
   */
  public void setFileList(ArrayList pFileList)
  {
    mFileList = pFileList;
  }

  /**
   * @param pFilePath
   *          The filePath to set.
   */
  public void setFilePath(String pFilePath)
  {
    mFilePath = pFilePath;
  }

  /**
   * @param pFilesets
   *          The filesets to set.
   */
  public void setFilesets(Vector pFilesets)
  {
    mFilesets = pFilesets;
  }

  protected void trim(File file) throws IOException
  {
    if (file != null)
    {
      try
      {
        long lastModified = file.lastModified();
        mRAF = new RandomAccessFile(file, "r");
        String s = file.getAbsolutePath();
        StringBuffer stringbuffer = new StringBuffer();
        String s1;
        while ((s1 = mRAF.readLine()) != null)
        {
          String s2 = s1.trim();
          if (s2.length() != 0)
          {
            stringbuffer.append(s2);

            if (!getPattern("<dsp[^>]*>").matcher(s2).matches()
                && !getPattern("</dsp[^>]*>").matcher(s2).matches()
                && !getPattern("<%@ [^>]*%>").matcher(s2).matches())
            {
              stringbuffer.append(NEW_LINE);
            }
          }
        }
        mRAF.close();
        file.delete();
        File newFile = new File(s);
        writeFile(newFile, stringbuffer.toString().getBytes());
        newFile.setLastModified(lastModified);
      }
      catch (IOException e)
      {
        if (mRAF != null) mRAF.close();
        log("Warning: " + e.getMessage());
      }
    }
  }

  protected void trim(File pBase, String pFiles[])
  {
    if (pFiles != null)
    {
      for (int i = 0; i < pFiles.length; i++)
      {
        try
        {
          trim(new File(pBase, pFiles[i]));
        }
        catch (IOException e)
        {
          log("Warning: " + e.getMessage());
        }
      }
    }
  }

  protected void writeFile(File pFile, byte pData[]) throws IOException
  {
    try
    {
      mRAF = new RandomAccessFile(pFile, "rw");
      mRAF.write(pData);
    }
    finally
    {
      mRAF.close();
    }
  }
}